@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2018	PgPool Global Development Group
+ * Copyright (c) 2003-2017	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -152,14 +152,6 @@ typedef struct {
 	/* If true, we are doing extended query message */
 	bool doing_extended_query_message;
 
-	/* If true, we have rewritten where_to_send map in the current query
-	 * context. pool_unset_query_in_progress() should restore the data from
-	 * where_to_send_save.  For now, this is only necessary while doing
-	 * extended query protocol and in streaming replication mode.
-	 */
-	bool need_to_restore_where_to_send;
-	bool where_to_send_save[MAX_NUM_BACKENDS];
-
 	/* If true, the command in progress has finished successfully. */
 	bool command_success;
 
@@ -229,12 +221,9 @@ typedef struct {
 	List *pending_messages;
 
 	/*
-	 * The last pending message. Reset at Ready for query.  Note that this is
-	 * a shallow copy of pending message.  Once the are is reset,
-	 * previos_message_exists is set to false.
+	 * The last pending message. Reset at Ready for query.
 	 */
-	bool previous_message_exists;
-	POOL_PENDING_MESSAGE previous_message;
+	POOL_PENDING_MESSAGE *previous_message;
 
 	/* Protocol major version number */
 	int major;
@@ -271,8 +260,6 @@ extern void pool_clear_sent_message_list(void);
 extern void pool_sent_message_destroy(POOL_SENT_MESSAGE *message);
 extern POOL_SENT_MESSAGE *pool_get_sent_message(char kind, const char *name, POOL_SENT_MESSAGE_STATE state);
 extern void pool_set_sent_message_state(POOL_SENT_MESSAGE *message);
-extern void pool_zap_query_context_in_sent_messages(POOL_QUERY_CONTEXT *query_context);
-extern POOL_SENT_MESSAGE * pool_get_sent_message_by_query_context(POOL_QUERY_CONTEXT * query_context);
 extern void pool_unset_writing_transaction(void);
 extern void pool_set_writing_transaction(void);
 extern bool pool_is_writing_transaction(void);
@@ -292,7 +279,7 @@ extern void pool_pending_messages_destroy(void);
 extern POOL_PENDING_MESSAGE *pool_pending_message_create(char kind, int len, char *contents);
 extern void pool_pending_message_free_pending_message(POOL_PENDING_MESSAGE *message);
 extern void pool_pending_message_dest_set(POOL_PENDING_MESSAGE* message, POOL_QUERY_CONTEXT *query_context);
-extern void pool_pending_message_query_context_dest_set(POOL_PENDING_MESSAGE* message, POOL_QUERY_CONTEXT *query_context);
+void pool_pending_message_query_context_dest_set(POOL_PENDING_MESSAGE* message, POOL_QUERY_CONTEXT *query_context);
 extern void pool_pending_message_query_set(POOL_PENDING_MESSAGE* message, POOL_QUERY_CONTEXT *query_context);
 extern void pool_pending_message_add(POOL_PENDING_MESSAGE* message);
 extern POOL_PENDING_MESSAGE *pool_pending_message_head_message(void);
@@ -304,13 +291,8 @@ extern void pool_pending_message_reset_previous_message(void);
 extern void pool_pending_message_set_previous_message(POOL_PENDING_MESSAGE *message);
 extern POOL_PENDING_MESSAGE *pool_pending_message_get_previous_message(void);
 extern bool pool_pending_message_exists(void);
-extern const char *pool_pending_message_type_to_string(POOL_MESSAGE_TYPE type);
-extern POOL_PENDING_MESSAGE *pool_pending_message_find_lastest_by_query_context(POOL_QUERY_CONTEXT *qc);
-extern int pool_pending_message_get_target_backend_id(POOL_PENDING_MESSAGE *msg);
-extern int	pool_pending_message_get_message_num_by_backend_id(int backend_id);
 extern void dump_pending_message(void);
 extern void pool_set_major_version(int major);
-extern int pool_get_major_version(void);
 extern void pool_set_minor_version(int minor);
 extern int pool_get_minor_version(void);
 extern void pool_set_preferred_master_node_id(int node_id);
