@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2019	PgPool Global Development Group
+ * Copyright (c) 2003-2015	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -28,6 +28,7 @@
 
 #include "config.h"
 #include "pool_type.h"
+#include "pcp/libpcp_ext.h"
 #include "utils/pool_signal.h"
 #include "parser/nodes.h"
 
@@ -235,6 +236,9 @@ typedef struct {
 	char *auth_arg;
 	char *database;
 	char *username;
+	ConnectionInfo *con_info; /* shared memory coninfo used
+						   * for handling the query containing
+						   * pg_terminate_backend*/
 } POOL_CONNECTION;
 
 /*
@@ -345,9 +349,8 @@ extern int my_master_node_id;
 
 #define REPLICATION (pool_config->replication_mode)
 #define MASTER_SLAVE (pool_config->master_slave_mode)
-#define STREAM (MASTER_SLAVE && !strcmp("stream", pool_config->master_slave_sub_mode))
-#define SL_MODE STREAM	/* for upper compatibility with 3.7 and later */
-#define SLONY (MASTER_SLAVE && !strcmp("slony", pool_config->master_slave_sub_mode))
+#define STREAM (MASTER_SLAVE && pool_config->master_slave_sub_mode == STREAM_MODE)
+#define SLONY (MASTER_SLAVE && pool_config->master_slave_sub_mode == SLONY_MODE)
 #define DUAL_MODE (REPLICATION || MASTER_SLAVE)
 #define RAW_MODE (!REPLICATION && !MASTER_SLAVE)
 #define MAJOR(p) (pool_get_major_version())
@@ -378,7 +381,7 @@ extern int my_master_node_id;
 											   * in stable state */
 #define MAX_IDENTIFIER_LEN		128
 
-#define SERIALIZE_ACCEPT (pool_config->serialize_accept != 0 && \
+#define SERIALIZE_ACCEPT (pool_config->serialize_accept == true && \
 						  pool_config->child_life_time == 0)
 
 /*
@@ -402,9 +405,12 @@ typedef enum {
 	PROMOTE_NODE_REQUEST
 } POOL_REQUEST_KIND;
 
+#define REQ_DETAIL_SWITCHOVER	0x00000001		/* failover due to switch over */
+
 typedef struct {
 	POOL_REQUEST_KIND	kind;		/* request kind */
-	unsigned int wd_failover_id;    /* watchdog ID for this failover operation */
+	unsigned char request_details;	/* option flags kind */
+	unsigned int wd_failover_id;	/* watchdog ID for this failover operation */
 	int node_id[MAX_NUM_BACKENDS];	/* request node id */
 	int count;						/* request node ids count */
 }POOL_REQUEST_NODE;
@@ -484,22 +490,6 @@ typedef enum
     EXITING
 } ProcessState;
 
-#define MAX_PG_VERSION_STRING	512
-
-/*
- * PostgreSQL version descriptor
- */
-typedef struct
-{
-	short	major;	/* major version number in up to 3 digits decimal.
-					 * Examples: 120, 110, 100, 96.
-					 */
-	short	minor;	/* minor version number in up to 2 digits decimal.
-					 * Examples: 0, 1, 2, 10, 23.
-					 */
-	char	version_string[MAX_PG_VERSION_STRING+1];	/* original version string */
-}			PGVersion;
-
 extern ProcessState processState;
 
 extern POOL_CONNECTION_POOL *pool_connection_pool;	/* connection pool */
@@ -514,7 +504,7 @@ extern volatile sig_atomic_t *InRecovery;
 extern char remote_ps_data[];		/* used for set_ps_display */
 extern volatile sig_atomic_t got_sighup;
 extern volatile sig_atomic_t exit_request;
-extern volatile sig_atomic_t ignore_sigusr1;
+
 
 #define QUERY_STRING_BUFFER_LEN 1024
 extern char query_string_buffer[];		/* last query string sent to simpleQuery() */
@@ -527,8 +517,8 @@ extern char remote_port[];	/* client port */
 /*
  * public functions
  */
-extern void register_watchdog_state_change_interrupt(void);
-extern bool register_node_operation_request(POOL_REQUEST_KIND kind, int* node_id_set, int count, unsigned int wd_failover_id);
+extern void register_watchdog_state_change_interupt(void);
+extern bool register_node_operation_request(POOL_REQUEST_KIND kind, int* node_id_set, int count, bool switch_over, unsigned int wd_failover_id);
 extern char *get_config_file_name(void);
 extern char *get_hba_file_name(void);
 extern void do_child(int *fds);
@@ -558,9 +548,9 @@ extern POOL_STATUS ErrorResponse(POOL_CONNECTION *frontend,
 extern void NoticeResponse(POOL_CONNECTION *frontend,
 								  POOL_CONNECTION_POOL *backend);
 
-extern void notice_backend_error(int node_id);
-extern void degenerate_backend_set(int *node_id_set, int count, unsigned int wd_failover_id);
-extern bool degenerate_backend_set_ex(int *node_id_set, int count, bool error, bool test_only, unsigned int wd_failover_id);
+extern void notice_backend_error(int node_id, bool switch_over);
+extern void degenerate_backend_set(int *node_id_set, int count, bool switch_over, unsigned int wd_failover_id);
+extern bool degenerate_backend_set_ex(int *node_id_set, int count, bool error, bool test_only, bool switch_over, unsigned int wd_failover_id);
 extern void promote_backend(int node_id, unsigned int wd_failover_id);
 extern void send_failback_request(int node_id, bool throw_error, unsigned int wd_failover_id);
 
@@ -621,7 +611,6 @@ extern void pool_shmem_exit(int code);
 
 extern void pool_semaphore_create(int numSems);
 extern void pool_semaphore_lock(int semNum);
-extern int	pool_semaphore_lock_allow_interrupt(int semNum);
 extern void pool_semaphore_unlock(int semNum);
 
 extern BackendInfo *pool_get_node_info(int node_number);
@@ -632,9 +621,9 @@ extern POOL_STATUS OneNode_do_command(POOL_CONNECTION *frontend, POOL_CONNECTION
 
 /* child.c */
 extern POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
-	int db_node_id, char *hostname, int port, char *dbname, char *user, char *password, bool retry);
+	char *hostname, int port, char *dbname, char *user, char *password, bool retry);
 extern POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection_noerror(
-    int db_node_id, char *hostname, int port, char *dbname, char *user, char *password, bool retry);
+    char *hostname, int port, char *dbname, char *user, char *password, bool retry);
 extern void discard_persistent_db_connection(POOL_CONNECTION_POOL_SLOT *cp);
 
 /* define pool_system.c */
@@ -665,7 +654,6 @@ extern void pool_ps_idle_display(POOL_CONNECTION_POOL *backend);
 extern void start_recovery(int recovery_node);
 extern void finish_recovery(void);
 extern int wait_connection_closed(void);
-extern int ensure_conn_counter_validity(void);
 
 /* child.c */
 extern void cancel_request(CancelPacket *sp);
@@ -676,7 +664,6 @@ extern int send_to_pg_frontend(char* data, int len, bool flush);
 extern int pg_frontend_exists(void);
 extern int set_pg_frontend_blocking(bool blocking);
 extern int get_frontend_protocol_version(void);
-extern PGVersion *Pgversion(POOL_CONNECTION_POOL * backend);
 
 /* pool_process_query.c */
 extern void reset_variables(void);
@@ -742,6 +729,5 @@ extern int pool_send_to_frontend(char* data, int len, bool flush);
 extern int pool_frontend_exists(void);
 extern pid_t pool_waitpid(int *status);
 extern int write_status_file(void);
-extern int get_next_master_node(void);
 
 #endif /* POOL_H */
