@@ -29,8 +29,6 @@
 #include "config.h"
 #include <sys/types.h>
 #include <sys/socket.h>
-#include "pcp/libpcp_ext.h"
-#include <stddef.h>
 #include "libpq-fe.h"
 /* Define common boolean type. C++ and BEOS already has it so exclude them. */
 #ifdef c_plusplus
@@ -59,21 +57,6 @@ typedef char bool;
 #endif /* not C++ */
 #endif /* __BEOS__ */
 
-/* ----------------------------------------------------------------
- *              Section 5:  offsetof, lengthof, endof, alignment
- * ----------------------------------------------------------------
- */
-/*
- * offsetof
- *      Offset of a structure/union field within that structure/union.
- *
- *      XXX This is supposed to be part of stddef.h, but isn't on
- *      some systems (like SunOS 4).
- */
-#ifndef offsetof
-#define offsetof(type, field)   ((long) &((type *)0)->field)
-#endif
-
 #define PointerIsValid(pointer) ((const void*)(pointer) != NULL)
 typedef signed char int8;		/* == 8 bits */
 typedef signed short int16;		/* == 16 bits */
@@ -89,20 +72,27 @@ typedef unsigned char uint8;	/* == 8 bits */
 typedef unsigned short uint16;	/* == 16 bits */
 typedef unsigned int uint32;	/* == 32 bits */
 
+#ifdef HAVE_LONG_INT_64
+/* Plain "long int" fits, use it */
 typedef long int int64;
+typedef unsigned long int uint64;
+
+#define pool_atoi64 atol
+#elif defined(HAVE_LONG_LONG_INT_64)
+/* We have working support for "long long int", use that */
+typedef long long int int64;
+typedef unsigned long long int uint64;
+#define pool_atoi64 atoll
+#else
+/* neither HAVE_LONG_INT_64 nor HAVE_LONG_LONG_INT_64 */
+#error must have a working 64-bit integer datatype
+#endif
 
 typedef enum {
 	LOAD_UNSELECTED = 0,
 	LOAD_SELECTED
 } LOAD_BALANCE_STATUS;
 
-/*
- * Backend status record file
- */
-typedef struct {
-	BACKEND_STATUS status[MAX_NUM_BACKENDS];
-} BackendStatusRecord;
-
 extern int assert_enabled;
 extern void ExceptionalCondition(const char *conditionName,
 								 const char *errorType,
@@ -220,7 +210,6 @@ typedef unsigned int AuthRequest;
 typedef uint8 bits8;			/* >= 8 bits */
 typedef uint16 bits16;			/* >= 16 bits */
 typedef uint32 bits32;			/* >= 32 bits */
-typedef unsigned long long int uint64;
 
 /*
  * stdint.h limits aren't guaranteed to be present and aren't guaranteed to
