@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2014	PgPool Global Development Group
+ * Copyright (c) 2003-2017	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -30,7 +30,6 @@
 #include "parser/parser.h"
 #include "parser/pg_list.h"
 #include "parser/parsenodes.h"
-#include "parallel_query/pool_rewrite_query.h"
 #include "context/pool_session_context.h"
 #include "utils/pool_process_reporting.h"
 
@@ -46,7 +45,6 @@ extern char *copy_null; /* copy null string */
 
 extern int is_select_pgcatalog;
 extern int is_select_for_update; /* also for SELECT ... INTO */
-extern bool is_parallel_table;
 extern char *parsed_query;
 
 /*
@@ -92,9 +90,6 @@ extern POOL_STATUS BindComplete(POOL_CONNECTION *frontend,
 extern POOL_STATUS CloseComplete(POOL_CONNECTION *frontend,
 								 POOL_CONNECTION_POOL *backend);
 
-extern POOL_STATUS CommandComplete(POOL_CONNECTION *frontend,
-								   POOL_CONNECTION_POOL *backend);
-
 extern POOL_STATUS ParameterDescription(POOL_CONNECTION *frontend,
 										POOL_CONNECTION_POOL *backend);
 
@@ -120,6 +115,8 @@ extern POOL_STATUS ProcessBackendResponse(POOL_CONNECTION *frontend,
 										  POOL_CONNECTION_POOL *backend,
 										  int *state, short *num_fields);
 
+extern void handle_query_context(POOL_CONNECTION_POOL *backend);;
+
 /*
  * modules defined in pool_proto2.c
  */
@@ -168,7 +165,6 @@ extern int detect_deadlock_error(POOL_CONNECTION *master, int major);
 extern int detect_serialization_error(POOL_CONNECTION *master, int major, bool unread);
 extern int detect_active_sql_transaction_error(POOL_CONNECTION *backend, int major);
 extern int detect_query_cancel_error(POOL_CONNECTION *backend, int major);
-extern int	detect_idle_in_transaction_sesion_timeout_error(POOL_CONNECTION * backend, int major);
 extern bool is_partition_table(POOL_CONNECTION_POOL *backend, Node *node);
 extern POOL_STATUS pool_discard_packet(POOL_CONNECTION_POOL *cp);
 extern void query_cache_register(char kind, POOL_CONNECTION *frontend, char *database, char *data, int data_len);
@@ -185,4 +181,9 @@ extern void read_kind_from_one_backend(POOL_CONNECTION *frontend, POOL_CONNECTIO
 extern void do_error_command(POOL_CONNECTION *backend, int major);
 extern void raise_intentional_error_if_need(POOL_CONNECTION_POOL *backend);
 
+/*
+ * modules defined in CommandComplete.c
+ */
+extern POOL_STATUS CommandComplete(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend, bool command_complete);
+
 #endif
