@@ -3,7 +3,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2016	PgPool Global Development Group
+ * Copyright (c) 2003-2017	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -88,7 +88,9 @@ static POOL_INTERNAL_BUFFER *pool_create_buffer(void);
 static void pool_discard_buffer(POOL_INTERNAL_BUFFER *buffer);
 static void pool_add_buffer(POOL_INTERNAL_BUFFER *buffer, void *data, size_t len);
 static void *pool_get_buffer(POOL_INTERNAL_BUFFER *buffer, size_t *len);
+#ifdef NOT_USED
 static char *pool_get_buffer_pointer(POOL_INTERNAL_BUFFER *buffer);
+#endif
 static char *pool_get_current_cache_buffer(size_t *len);
 static size_t pool_get_buffer_length(POOL_INTERNAL_BUFFER *buffer);
 static void pool_check_and_discard_cache_buffer(int num_oids, int *oids);
@@ -2996,6 +2998,7 @@ static size_t pool_get_buffer_length(POOL_INTERNAL_BUFFER *buffer)
 	return buffer->buflen;
 }
 
+#ifdef NOT_USED
 /*
  * Get internal buffer pointer.
  */
@@ -3005,7 +3008,7 @@ static char *pool_get_buffer_pointer(POOL_INTERNAL_BUFFER *buffer)
 		return NULL;
 	return buffer->buf;
 }
-
+#endif
 /*
  * Get query cache buffer struct of current query context
  */
@@ -3480,7 +3483,6 @@ long long int pool_stats_count_up_num_cache_hits(void)
 
 static volatile POOL_HASH_HEADER *hash_header;
 static volatile POOL_HASH_ELEMENT *hash_elements;
-static volatile POOL_HASH_ELEMENT hash_free_body;
 static volatile POOL_HASH_ELEMENT *hash_free;
 
 /*
@@ -3888,7 +3890,7 @@ POOL_SHMEM_STATS *pool_get_shmem_storage_stats(void)
 	mystats.cache_stats.num_selects = stats->num_selects;
 	mystats.cache_stats.num_cache_hits = stats->num_cache_hits;
 
-	if (pool_config->memqcache_method == SHMEM_CACHE)
+	if (pool_config->memqcache_method != SHMEM_CACHE)
 		return &mystats;
 
 	/* number of total hash entries */
@@ -3959,16 +3961,6 @@ void forward_pending_data(POOL_CONNECTION *frontend, POOL_CONNECTION *backend)
 	int len;
 	char *buf;
 
-	/* If are doing extended query and we are in streaming replication
-	 * mode, we need to retrieve any response from backend.
-	 */
-	if (!pool_is_pending_response())
-	{
-		ereport(DEBUG1,
-				(errmsg("forward_pending_data: no peding data")));
-		return;
-	}
-
 	/* Send flush messsage to backend to retrieve response of backend */
 	pool_write(backend, "H", 1);		
 	len = htonl(sizeof(len));
